//Autor: Peter Reimann
//Datum: 06.07.2004
//Programmiersprache: Java2 1.4
//Beschreibung: Das Serverprogramm.
//              Es bekommt ber einen InputStream das Programm, das der Client
//              ausfhren mchte und fhrt dieses dann fr ihn aus.
//              Das Ergebnis des Programms wird ber einen PrintSteam
//              wieder zum Client zurckgeschickt.
//              folgende Programme sind zulssig:
//              highscoreread: liest die highscoredatei aus
//              highscoreput: schreibt einen neuen Eintrag in die highscoredatei
//              loesung_sol: berechnet die Lsung frs Solitaire
//              loesung_schiebe: berechnet die Lsung frs Schiebespiel
//
//              Da man, den Server mit Hilfe von Threads implementiert hat,
//              knnen mehrere Clients gleichzeitig darauf zugreifen.
//              Um gleichzeitige Zugriffe auf die gleichen Dateien zu vermeiden,
//              wurde eine Art Warteschleife mit Hilfe des synchronized-Konstrukts verwendet.
//              Es gibt je Datei auf die zugegriffen werden kann(Solitaire, Schiebespiel, Highscore)
//              eine eigene synchronisierte Funktion. Benutzt ein Client nun die eine dieser
//              Funktionen, so wird diese Funktion, und zwar nur diese, fr andere Clients gesperrt.

import java.net.*;
import java.io.*;
 
 public class Server
 {
     public static ServerSocket serversocket;


     //Baut Serversocket auf und startet den Empfangsmodus
   public static void main(String[] args)
   {
     try {

       int port = config.port;

       if (args.length > 0)
          port = Integer.valueOf(args[0]).intValue();
       //man kann ber die Kommandozeile einen anderen Port angeben

       serversocket = new ServerSocket(port);

       while (true) {
         Socket socket = serversocket.accept();
         (new ClientThread( socket)).start();
       }
     } catch (IOException e) {
       System.err.println(e.toString());
       System.exit(1);
     }
   }
 }


  class Sol
  {
     //Synchronisierte Funktion fr die Berechnung der Lsung bei Solitaire
     synchronized static String execute(String s) throws IOException
     {

         int Zeichen;
         StringBuffer Ausgabe = new StringBuffer();

         Process p = Runtime.getRuntime().exec("java " + s);
         //Das Programm wird ausgefhrt und dessen Ausgabe in p_in gespeichert

         InputStream p_in = p.getInputStream();

         while ((Zeichen = p_in.read()) != -1)
         {
             Ausgabe = Ausgabe.append((char) Zeichen);
         }
         //Der Inhalt von p_in wird in Ausgabe geschrieben.

         return Ausgabe.toString();

     }

  }


  class Schiebe
  {
     //Synchronisierte Funktion fr die Berechnung der Lsung beim Schiebespiel
     synchronized static String execute(String s) throws IOException
     {

         int Zeichen;
         StringBuffer Ausgabe = new StringBuffer();

         Process p = Runtime.getRuntime().exec("java " + s);
         //Das Programm wird ausgefhrt und dessen Ausgabe in p_in gespeichert

         InputStream p_in = p.getInputStream();

         while ((Zeichen = p_in.read()) != -1)
         {
             Ausgabe = Ausgabe.append((char) Zeichen);
         }
         //Der Inhalt von p_in wird in Ausgabe geschrieben.

         return Ausgabe.toString();

     }
  }


  class Highscore
  {
     //Synchronisierte Funktion fr die Ausfhrung der Highscoreprogramme
     synchronized static String execute(String s) throws IOException
     {

         int Zeichen;
         StringBuffer Ausgabe = new StringBuffer();

         Process p = Runtime.getRuntime().exec("java " + s);
         //Das Programm wird ausgefhrt und dessen Ausgabe in p_in gespeichert

         InputStream p_in = p.getInputStream();

         while ((Zeichen = p_in.read()) != -1)
         {
             Ausgabe = Ausgabe.append((char) Zeichen);
         }
         //Der Inhalt von p_in wird in Ausgabe geschrieben.

         return Ausgabe.toString();

     }
  }
 

 class ClientThread
 extends Thread
 {
   private Socket socket;
 
   public ClientThread(Socket socket)
   {
     this.socket = socket;
   }
 

   public void run()
   {
     System.out.println("New connection");
     try {
       InputStream in = socket.getInputStream();
       PrintStream out = new PrintStream(socket.getOutputStream(), true);

       StringBuffer Eingabe = new StringBuffer();
       String Ausgabe = "";
       int Zeichen;

       while ( true )
       {  Zeichen = in.read();
	  if( ((char) Zeichen) == '\n')
	      {break;}
          Eingabe = Eingabe.append((char) Zeichen);
       }
       //Eingabe wird gelesen

       String s = Eingabe.toString();
       System.out.println(s);


       //berprfung der Eingabe auf zulssige Zeichen
	for (int i = 0; i < s.length(); i++) {
		char c = s.charAt(i);
		if (!(c >= 'a' && c <= 'z') && !(c >= 'A' && c <= 'Z')
			&& !(c >= '0' && c <= '9') && c != ' ' && c != '_')
		{
			System.out.println("Input rejected!");
		        socket.close();
			return;
		}
	}

        //Handelt es sich um zulssige Programme?
       if ( (s.regionMatches(0, "loesung_sol ", 0, 12))   || 
            (s.regionMatches(0, "highscoreput ", 0, 13))  ||
	    (s.regionMatches(0, "highscoreread", 0, 13)) ||
	    (s.regionMatches(0, "loesung_schiebe ", 0, 16))){


           //Je nach Eingabe wird die entsprechende synchronisierte Funktion aufgerufen
	   if (s.regionMatches(0, "loesung_sol ", 0, 12))
	       {
                   Ausgabe = Sol.execute(s);
               }
	   else
	   if (s.regionMatches(0, "loesung_schiebe ", 0, 16))
	       {
                   Ausgabe = Schiebe.execute(s);
               }
	   else
	   if ( (s.regionMatches(0, "highscoreput ", 0, 13))  ||
	        (s.regionMatches(0, "highscoreread", 0, 13)))
	       {
                   Ausgabe = Highscore.execute(s);
               }

       }

       System.out.println(Ausgabe);

       out.println(Ausgabe);
       //Die Ausgabe des Programms wird an den Client bergeben.

       socket.close();
       System.out.println("Connection closed");

     } catch (IOException e) {
       System.err.println(e.toString());
     }
   }
 }
